#!/usr/bin/python
# -*- coding: utf-8 -*-

################################################################################################################
# This script is a simple management system for series made of exercises and solution.                         #
# It is possible to make zipped series for moodle, a zip containing all series. Furthermore one can            #
# generate previews for one exercise/solution. Two handy functions are the make-workbook and the               #
# make-catalogue. The former one creaets a pdf containig all series, each one followed by its solution         #
# just like they were distributed. The latter one create a sort of index of all available exercises in         #
# the system. Each exercise is followed by its solution.                                                       #
#                                                                                                              #
# The structure for a new exercise should be created by using the make-new-exercise function.                  #
# For further help, please refer to the help function of the software.                                         #
# -------------------------------------------------------------------------------------------------------------#
#                                                                                                              #
# Author: Andreas Ruppen                                                                                       #
# Licensed under the Apache License, Version 2.0 (the "License");                                              #
#   you may not use this file except in compliance with the License.                                           #
#   You may obtain a copy of the License at                                                                    #
#                                                                                                              #
#       http://www.apache.org/licenses/LICENSE-2.0                                                             #
#                                                                                                              #
#   Unless required by applicable law or agreed to in writing, software                                        #
#   distributed under the License is distributed on an "AS IS" BASIS,                                          #
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                   #
#   See the License for the specific language governing permissions and                                        #
#   limitations under the License.                                                                             #
################################################################################################################
import sys
import os
import getopt
import shutil
import logging.config

if float(sys.version[:3])<3.0:
    import ConfigParser
else: 
    import configparser as ConfigParser
import subprocess
from subprocess import STDOUT
try:
    from subprocess import DEVNULL # py3k
except ImportError:
    import os
    DEVNULL = open(os.devnull, 'wb')
from pkg_resources import resource_filename
from distutils.dir_util import copy_tree
from os.path import dirname, join, expanduser
from utils import Utils, ZipUtils
from utils.LaTeX import *

class SMS:
    def __init__(self):

        self.__INSTALL_DIR = dirname(__file__)
        self.__CONFIG_DIR = '/etc/SeriesManagementSystem/'
        logging.basicConfig(level=logging.ERROR)
        logging.config.fileConfig(
            [join(self.__CONFIG_DIR, 'logging.conf'), expanduser('~/.logging.conf'), 'logging.conf'])
        self.__log = logging.getLogger('seriesManagementSystem')
        
        self.__cwd = os.getcwd()
        self.__keepTempFiles = False
        
        self.__log.debug("\033[1;33m"+self.__cwd+"\033[0m")
        self.__exoStructure = ["code", "code/donne", "code/solution", "latex", "latex/ressources", "latex/ressources/figures", "latex/ressources/code"]

        smsConfig = ConfigParser.SafeConfigParser()
        self.__log.debug("Reading general configuration from lecture.cfg")
        smsConfig.read([join(resource_filename(__name__, 'data'), 'lecture.cfg'), "lecture.cfg"])
        self.__smscmoodleOutputDir = smsConfig.get("Config", "moodleOutputDir")
        self.__smscremoveUnzipped = smsConfig.getboolean("Config", "removeUnzipped")
        self.__smscdozipfiles = smsConfig.getboolean("Config",  "createZip")
        self.__smscupdateBibTex = smsConfig.getboolean("Config", "updateBibTex")
        self.__smscopencmd = smsConfig.get("Config", "opencmd")
        self.__smcsdebuglevel = smsConfig.getint("Config", "debugLevel")
        if smsConfig.has_option("Config", "addClearPage"):
            self.__smscaddClearPage = smsConfig.getboolean("Config", "addClearPage")
        else:
            self.__smscaddClearPage = False
        self.__usepdftk = smsConfig.getboolean("Config", "usepdftk")
        self.__exoDirName = smsConfig.get("Config", "exoDirName")
        self.__seriesConfigDir = smsConfig.get("Config", "seriesConfigDir")
        self.__outputDir = smsConfig.get("Config", "outputDir")
        self.__smscname = smsConfig.get("Lecture", "name")
        self.__smsclecturer = smsConfig.get("Lecture", "lecturer")
        self.__smscyear = smsConfig.get("Lecture", "year")
        self.__smscexercisetext = smsConfig.get("Lecture", "exercisetext")
        self.__smscsolutiontext = smsConfig.get("Lecture", "solutiontext")
        self.__smsccontenttext = smsConfig.get("Lecture", "contenttext")
        self.__smscheadertitle = smsConfig.get("Lecture", "headertitle")
        self.__smscbibtex = smsConfig.get("Lecture", "bibtex")
        self.__noCiteList = smsConfig.get("Lecture", "nocite").split(",")
        self.__smscunilogo = smsConfig.get("Logo", "unilogo")
        self.__smscgroupelogo = smsConfig.get("Logo", "groupelogo")
        self.__smscpdfkeyword = smsConfig.get("PDF", "pdfkeyword")
        self.__smscpdftitle = smsConfig.get("PDF", "pdftitle")
        self.__smscpdfauthor = smsConfig.get("PDF", "pdfauthor")
        self.__latex_error_messages = (
                                "Type X to quit or <RETURN> to proceed",
                                "! Undefined control sequence.",
                                "? ",
                                "Type  H <return>  for immediate help.",
                                "Enter file name: ",
                                "or enter new name. (Default extension: sty)"
                                )
        self.__latex_recompile_messages = (
                                    "recompile ",
                                    "re-run ",
                                    "undefined references.",
                                    "rerun "
                                    )
        self.__serie = -1
        self.__exercise = -1
        self.__exclude_from_zip = set(['nbproject'])

    def createNewExercice(self):
        self.__exercise = Utils.nextUnusedExercice()
        self.__log.debug("Creating Exercice Structure %s", self.__exercise)
        os.chdir(self.__cwd)
        os.mkdir(self.__exoDirName+"/"+"ex"+str(self.__exercise))
        for adir in self.__exoStructure:
            os.mkdir(self.__exoDirName+"/"+"ex"+str(self.__exercise)+"/"+adir)
        extex = open(self.__exoDirName+"/"+"ex"+str(self.__exercise)+"/latex/exo"+str(self.__exercise)+".tex", 'w')
        soltex = open(self.__exoDirName+"/"+"ex"+str(self.__exercise)+"/latex/exo"+str(self.__exercise)+"sol.tex", 'w')
        extex.write("\exercice{}\n")
        extex.write("voir le site \cite{WEBT} et \cite{T03}")
        soltex.write("\exercice{}\n")
        soltex.write("voir le site \cite{WEBT} et \cite{T03}")
        extex.close()
        soltex.close()

    def buildSerie(self):
        if self.__smscupdateBibTex:
            Utils.doUpdateBibTex(self.__smscbibtex, self.__noCiteList)
        seriesConfig = ConfigParser.SafeConfigParser()
        self.__log.debug(self.__seriesConfigDir+"/serie"+str(self.__serie)+".cfg")
        seriesConfig.read(self.__seriesConfigDir+"/serie"+str(self.__serie)+".cfg")
        titles = seriesConfig.get('Serie', 'titles')
        numbers = seriesConfig.get('Serie', 'exo-numbers')
        #draft = seriesConfig.getboolean('Serie', 'draft')
        # check if dir exists with os.path.isdir
        if os.path.isdir(os.path.join(self.__outputDir, self.__smscmoodleOutputDir+str(self.__serie))):
            shutil.rmtree(os.path.join(self.__outputDir, self.__smscmoodleOutputDir+str(self.__serie)))
        os.mkdir(os.path.join(self.__outputDir, self.__smscmoodleOutputDir+str(self.__serie)))
        os.mkdir(os.path.join(os.path.join(self.__outputDir, self.__smscmoodleOutputDir+str(self.__serie)),'donne'))
        os.mkdir(os.path.join(os.path.join(self.__outputDir, self.__smscmoodleOutputDir+str(self.__serie)),'solution'))

        outputDir=os.path.join(os.path.join(self.__outputDir, self.__smscmoodleOutputDir+str(self.__serie)),'donne')
        self.__doCreateSerie(titles.split(','), numbers.split(','), outputDir)
        outputDir =  os.path.join(os.path.join(self.__outputDir, self.__smscmoodleOutputDir+str(self.__serie)),'solution')
        self.__doCreateSolution(titles.split(','), numbers.split(','), outputDir)
        self.__addCodeDonne(numbers.split(","))
        self.__addCodeSolution(numbers.split(","))

        Utils.cleanTempFiles([])

    def __doCreateSerie(self, _titles, _numbers, _outputDir):
        texfile = "/tmp/serie"+str(self.__serie)+".tex"
        serie = open(texfile, 'w') #use open(file 'a') for appending to a given file
        self.__smscsolutiontext = ''
        latex = LaTeX(self.__serie)
        latex.createHeader(serie, _titles)

        for number in _numbers:
            serie.write(r"\newpage"+"\n")
            serie.write(r'\renewcommand{\includepath}{\compilationpath/'+self.__exoDirName+'/ex'+number+'/latex/ressources}'+'\n')
            exo = open(self.__exoDirName+"/"+"ex"+number+"/latex/exo"+number+".tex", 'r')
            for line in exo:
                serie.write(line)
            serie.write(r"\clearpage"+"\n")

        latex.createFooter(serie)
        serie.close()

        Utils.doLatex(texfile, _outputDir)


    def __doCreateSolution(self, _titles, _numbers, _outputDir):
        texfile = "/tmp/solution"+str(self.__serie)+".tex"
        solution = open(texfile, 'w')
        latex = LaTeX(self.__serie)
        latex.createHeader(solution, _titles)

        for number in _numbers:
            solution.write(r"\newpage"+"\n")
            solution.write(r'\renewcommand{\includepath}{\compilationpath/'+self.__exoDirName+'/ex'+number+'/latex/ressources}'+'\n')
            exo = open(self.__exoDirName+"/"+"ex"+number+"/latex/exo"+number+"sol.tex", 'r')
            for line in exo:
                solution.write(line)

        latex.createFooter(solution)
        solution.close()

        Utils.doLatex(texfile, _outputDir)

    def __addCodeDonne(self, _exonumbers):
        """Add the code skeletons which are given wich each exercise"""
        self.__log.info("Adding source code for donnee")
        for number in _exonumbers:
            #ZipUtils.myZip(self.__exoDirName+"/"+"ex"+number+"/code/donne", os.path.join(os.path.join(self.__outputDir,self.__smscmoodleOutputDir+str(self.__serie)),'donne/donnee_s'+str(self.__serie)+'_e'+number+'.zip'), 'donnee_s'+str(self.__serie)+'_e'+number)
            ZipUtils.myTar(self.__exoDirName+"/"+"ex"+number+"/code/donne", os.path.join(os.path.join(self.__outputDir,self.__smscmoodleOutputDir+str(self.__serie)),'donne/donnee_s'+str(self.__serie)+'_e'+number+'.tar.gz'), 'donnee_s'+str(self.__serie)+'_e'+number)
            #ZipUtils.sysTar(self.__exoDirName+"/"+"ex"+number+"/code/donne", os.path.join(os.path.join(self.__outputDir,self.__smscmoodleOutputDir+str(self.__serie)),'donne/donnee_s'+str(self.__serie)+'_e'+number+'.tar.gz'), 'donnee_s'+str(self.__serie)+'_e'+number)

    def __addCodeSolution(self, _exonumbers):
        """Adds code solutions, if any"""
        self.__log.info("Adding source code for solution")
        for number in _exonumbers:
            #ZipUtils.myZip(self.__exoDirName+"/"+"ex"+number+"/code/solution", os.path.join(os.path.join(self.__outputDir, self.__smscmoodleOutputDir+str(self.__serie)), 'solution/solution_s'+str(self.__serie)+'_e'+number+'.zip'), 'solution_s'+str(self.__serie)+'_e'+number)
            ZipUtils.myTar(self.__exoDirName+"/"+"ex"+number+"/code/solution", os.path.join(os.path.join(self.__outputDir, self.__smscmoodleOutputDir+str(self.__serie)), 'solution/solution_s'+str(self.__serie)+'_e'+number+'.tar.gz'), 'solution_s'+str(self.__serie)+'_e'+number)
            #ZipUtils.sysTar(self.__exoDirName+"/"+"ex"+number+"/code/solution", os.path.join(os.path.join(self.__outputDir, self.__smscmoodleOutputDir+str(self.__serie)), 'solution/solution_s'+str(self.__serie)+'_e'+number+'.tar.gz'), 'solution_s'+str(self.__serie)+'_e'+number)

    def buildAllSeries(self):
        """Builds all configured Series"""
        seriesConfigFiles = os.listdir(self.__seriesConfigDir)
        seriesConfigFiles = Utils.natsort(seriesConfigFiles)
        #seriesConfigFiles.sort()
        if os.path.isdir(self.__smscmoodleOutputDir):
            shutil.rmtree(self.__smscmoodleOutputDir)
        os.mkdir(self.__smscmoodleOutputDir)
        self.__outputDir = self.__smscmoodleOutputDir
        for config in seriesConfigFiles:
            if not config.startswith("."):
                self.__log.debug("Will treat from file: "+config+" serie:"+config.split(".")[0].partition("serie")[2])
                self.__serie=config.split(".")[0].partition("serie")[2]
                self.__log.info("Found Serie "+self.__serie+". Will now build it.")
                self.buildSerie()


    def makeWorkbook(self):
        seriesConfigFiles = os.listdir(self.__seriesConfigDir)
        #seriesConfigFiles.sort()
        seriesConfigFiles = Utils.natsort(seriesConfigFiles)
        if os.path.isdir(self.__smscmoodleOutputDir):
            shutil.rmtree(self.__smscmoodleOutputDir)
        os.mkdir(self.__smscmoodleOutputDir)
        self.__outputDir = self.__smscmoodleOutputDir
        for config in seriesConfigFiles:
            self.__serie=int(config.split(".")[0].partition("serie")[2])
            self.__log.info("Found Serie "+str(self.__serie)+". Will now build it.")
            seriesConfig = ConfigParser.SafeConfigParser()
            self.__log.debug("Reading "+self.__seriesConfigDir+"/serie"+str(self.__serie)+".cfg")
            seriesConfig.read(self.__seriesConfigDir+"/"+config)
            titles = seriesConfig.get('Serie', 'titles')
            numbers = seriesConfig.get('Serie', 'exo-numbers')
            outputDir=self.__smscmoodleOutputDir
            self.__doCreateSerie(titles.split(','), numbers.split(','), outputDir)
            outputDir =  self.__smscmoodleOutputDir
            self.__doCreateSolution(titles.split(','), numbers.split(','), outputDir)
        self.__makeWorkBookTitlePage(outputDir)
        if self.__usepdftk:
            subprocess.call(["pdftk "+outputDir+"/*.pdf cat output workbook.pdf"], shell=True, cwd="./", stdout=DEVNULL, stderr=STDOUT)
        else:
            subprocess.call(["gs -q -dNOPAUSE -dBATCH -sDEVICE=pdfwrite -sOutputFile=workbook.pdf "+outputDir+"/*.pdf"], shell=True, cwd="./", stdout=DEVNULL, stderr=STDOUT)
        shutil.rmtree(self.__smscmoodleOutputDir)
        Utils.cleanTempFiles([])


    def __makeWorkBookTitlePage(self, _outputDir):
        texfile = "/tmp/0wbtitlepage.tex"
        wbtitle = open(texfile, 'w')
        latex = LaTeX(self.__serie)
        latex.makeWorkBookTitlePageHeader(wbtitle)
        seriesConfigFiles = os.listdir(self.__seriesConfigDir)
        #seriesConfigFiles.sort()
        seriesConfigFiles = Utils.natsort(seriesConfigFiles)
        for config in seriesConfigFiles:
            seriesConfig = ConfigParser.SafeConfigParser()
            seriesConfig.read(self.__seriesConfigDir+"/"+config)
            titles = seriesConfig.get('Serie', 'titles')
            numbers = seriesConfig.get('Serie', 'exo-numbers')
            serienumber = config.split(".")[0].partition("serie")[2]
            wbtitle.write(r"\textsf{ \textbf{S{\'e}rie "+serienumber+"}} \dotfill"+"\n")
            for number in numbers.split(","):
                wbtitle.write(number+"\n")
            wbtitle.write(r"\begin{itemize}"+"\n")
            for title in titles.split(","):
                wbtitle.write(r"\item "+title+"\n")
            wbtitle.write(r"\end{itemize}"+"\n")
        latex.printWorkBookTitlePageFooter(wbtitle)
        wbtitle.close()
        Utils.doLatex(texfile, _outputDir, True)

    def makeCatalogue(self):
        """Creates a pdf containing all exercises. Each exercise is always followed by its solution"""
        afile = "/tmp/catalogue.tex"
        self.__serie=0
        if os.path.isdir("Catalogue"):
            shutil.rmtree("Catalogue")
        os.mkdir("Catalogue")
        catalogue = open(afile, 'w') #use open(file 'a') for appending to a given file
        latex = LaTeX(self.__serie)
        latex.createHeader(catalogue, [])
        catalogue.write(r'\renewcommand{\exercice}[1]{\subsection*{Problem: #1}}'+"\n")
        catalogue.write(r'\renewcommand{\solution}[1]{\subsection*{Solution: #1}}'+"\n")
        catalogue.write(r'\renewcommand{\question}[1]{\subsubsection*{#1}}'+"\n")
        catalogue.write(r''+"\n")
        catalogue.write(r'\makeatletter'+"\n")
        catalogue.write(r'\renewcommand{\section}{\@startsection{section}{3}{2pt}{12pt}{10pt}{\center \huge \sffamily \bfseries}}'+"\n")
        catalogue.write(r'\renewcommand{\thesection}{(\roman{section})}'+"\n")
        catalogue.write(r'\renewcommand{\thesubsection}{(\roman{subsection})}'+"\n")
        exos = os.listdir(self.__exoDirName)
        #exos.sort()
        exos = Utils.natsort(exos)
        for exo in exos:
            if exo.find("ex") != -1:
                number = exo[2:]
                catalogue.write(r'\section*{Exercise '+number+'}'+"\n")
                catalogue.write(r'\renewcommand{\includepath}{\compilationpath/'+self.__exoDirName+'/ex'+number+'/latex/resources}'+'\n')
                exo = open(os.path.join(os.path.join(self.__exoDirName, "ex"+number),"latex/exo"+number+".tex"), 'r')
                for line in exo:
                    catalogue.write(line)
                exo.close()
                catalogue.write(r'\renewcommand{\includepath}{\compilationpath/'+self.__exoDirName+'/ex'+number+'/latex/resources}'+'\n')
                solution = open(os.path.join(os.path.join(self.__exoDirName, "ex"+number),"latex/exo"+number+"sol.tex"), 'r')
                for line in solution:
                    catalogue.write(line)
                solution.close()
                if self.__smscaddClearPage:
                    catalogue.write("\clearpage")


        latex.createFooter(catalogue)
        catalogue.close()
        outputDir = "Catalogue"
        Utils.doLatex(afile, outputDir, True)

    def previewExercice(self):
        self.__serie = 0
        self. __doCreateSerie([], [str(self.__exercise)], "/tmp/")
        if self.__smscopencmd.find(",") == -1:
            cmd = self.__smscopencmd
            arg = [cmd, "/tmp/serie0.pdf"]
        else:
            cmd = self.__smscopencmd.split(",")[0]
            arg = self.__smscopencmd.split(",")[1:]
            arg.append("/tmp/serie0.pdf")
            arg.insert(0, cmd)
        subprocess.Popen(cmd+" /tmp/serie0.pdf", shell=True)

    def previewSolution(self):
        self.__serie = 0
        self. __doCreateSolution([], [str(self.__exercise)], "/tmp/")
        if self.__smscopencmd.find(",") == -1:
            cmd = self.__smscopencmd
            arg = [cmd, "/tmp/solution0.pdf"]
        else:
            cmd = self.__smscopencmd.split(",")[0]
            arg = self.__smscopencmd.split(",")[1:]
            arg.append("/tmp/solution0.pdf")
            arg.insert(0, cmd)
        subprocess.Popen(cmd+" /tmp/solution0.pdf", shell=True)


    def createNewLecture(self, lecturename):
        """Create the directory structure for a new lecture"""
        if os.path.exists(lecturename):
            self.__log.critical("This lecture already exists. Please choose another name")
            return -1
        os.mkdir(lecturename)
        os.mkdir(join(lecturename, 'Exercises'))
        os.mkdir(join(lecturename, 'Series_properties'))
        f = open(join(join(lecturename, 'Series_properties'), 'serie1.cfg'),'w')
        f.write('[Serie]\n')
        f.write('titles: Classes et ADT, Programmation orient\'ee objets en Java - types statique et dynamique, Java: h\'eritage - polymorphisme - interfaces - ...\n')
        f.write('exo-numbers: 3,1,2\n')
        f.close()
        self.__log.debug(resource_filename(__name__, 'data'))
        copy_tree(resource_filename(__name__, 'data'),
                        lecturename)

    def usage(self):
        print ('Usage:')
        print (os.path.basename(sys.argv[0])+' [option] <command>') #sys.argv[0]
        print ('\033[1;33mWhere option is one of:\033[0m')
        print ('    -e for specifying an exercise')
        print ('    -s for specifying a serie')
        print ('    -u for updating/or not last visited date in bibtex')
        print ('    -k for keeping/or not unzipped files')
        print ('    -t for keeping temporary files in /tmp')
        print ('    -z for doing a zip file')
        print ('    -l lecture name')
        print ('\033[1;33mWhere command is one of:\033[0m')
        print ('    make-new-exercise.........................Creates a new exercise structure')
        print ('    build-serie (-s option mandatory).........Builds all for the specified serie and packs it for moodle')
        print ('    build-all-series..........................Builds all available series and packs them for moodle')
        print ('    make-workbook.............................Creates one big pdf wich contains all concatenated series')
        print ('    make-catalogue............................Creates a pdf containing all exercices')
        print ('    preview-exercise (-e option mandatory)....Previews the specified exercise')
        print ('    preview-solution (-e option mandatory)....Previews the solution for the specified exercise')
        print ('    make-new-lecture..........................Creates the directory structure for a new Lecture')


    def getArguments(self, argv):
        # Parse the command line options
        if len(argv) == 0:
            self.usage()
            sys.exit(3)
        try:
            options, args = getopt.getopt(argv, "e:s:huktzl:", ["make-new-lecture", "make-new-exercise", "build-serie", "build-all-series", "make-workbook", "make-catalogue", "preview-exercise", "preview-solution", "--help"])
        except getopt.GetoptError:
            self.usage()
            sys.exit(2)
        Utils.cleanDSStore("./")
        Utils.doCheckInstall()
        self.__log.debug("Parsing options")
        for option, arg in options:
            self.__log.debug("Passed options are  %s  and args are %s", option, arg)

            if option in ["-e"]:
                self.__log.info("Current exercise is: %s", arg)
                self.__exercise=int(arg)
            elif option in ["-s"]:
                self.__log.info("Current serie is: %s", arg)
                self.__serie=int(arg)
            if option in ["-u"]:
                if self.__smscupdateBibTex:
                    self.__smscupdateBibTex=False
                else:
                    self.__log.info("Updating Bibtex Last visited date")
                    self.__smscupdateBibTex=True
            if option in ["-k"]:
                if self.__smscremoveUnzipped:
                    self.__smscremoveUnzipped = False
                else:
                    self.__log.info("Keeping unzipped files")
                    self.__smscremoveUnzipped = True
            if option in ["-t"]:
                self.__keepTempFiles = True
            if option in ["-z"]:
                self.__log.info("Will zip the files")
                self.__smscdozipfiles = True
            if option in ["-l"]:
                lecturename = arg
        self.__log.debug("Parsing arguments")
        for option, arg in options:
            self.__log.debug("Passed options are  \"%s\"  and args are \"%s\"", option, arg)
            if option in ["--make-new-exercise"]:
                self.__log.info("Creating a new Exercice")
                self.createNewExercice()
                break
            elif option in ["--build-serie"]:
                if self.__serie == -1:
                    self.__serie = int(raw_input ("Which serie do you want to build? "))
                self.__log.info("Building Serie %s", self.__serie)
                self.buildSerie()
                if self.__smscdozipfiles:
                    self.__log.info("Zipping "+self.__smscmoodleOutputDir+str(self.__serie)+" into "+self.__smscmoodleOutputDir+str(self.__serie)+'.zip')
                    ZipUtils.myZip(self.__smscmoodleOutputDir+str(self.__serie), self.__smscmoodleOutputDir+str(self.__serie)+'.zip', self.__smscmoodleOutputDir+str(self.__serie))
                    if self.__smscremoveUnzipped:
                        shutil.rmtree(self.__smscmoodleOutputDir+str(self.__serie))
                    break
            elif option in ["--build-all-series"]:
                self.__log.info("Building All Available Series")
                self.buildAllSeries()
                if self.__smscdozipfiles:
                    self.__log.info("Zipping "+self.__smscmoodleOutputDir+" into "+self.__smscmoodleOutputDir+'.zip')
                    ZipUtils.myZip(self.__smscmoodleOutputDir, self.__smscmoodleOutputDir+'.zip', self.__smscmoodleOutputDir)
                    if self.__smscremoveUnzipped:
                        shutil.rmtree(self.__smscmoodleOutputDir)
                    break
            elif option in ["--make-workbook"]:
                self.__log.info("Building Workbook")
                self.makeWorkbook()
                break
            elif option in ["--make-catalogue"]:
                self.__log.info("Creating Catalogue of available Exercices")
                self.makeCatalogue()
                break
            elif option in ["--preview-exercise"]:
                if self.__exercise == -1:
                    self.__exercise = int(raw_input ("Which exercise do you want to preview? "))
                self.__log.info("Previewing exercise %s", self.__exercise)
                self.previewExercice()
                break
            elif option in ["--preview-solution"]:
                if self.__exercise == -1:
                    self.__exercise = int(raw_input ("Which solution do you want to preview? "))
                self.__log.info("Previewing solution %s", self.__exercise)
                self.previewSolution()
                break
            elif option in ["--make-new-lecture"]:
                self.createNewLecture(lecturename)
            elif option in ["--help", "-h"]:
                self.usage()
                break
        
class checkInstallException(Exception):
    """Used for raising exception during doCheckInstall of SMS class"""
    def __init__(self, missingProg):
        message = ""
        for prog in missingProg:
            message += prog
            message += "; "
        message = message[0:len(message)-2]
        self.missing = message

if __name__ == "__main__":
    sms = SMS()
    sms.getArguments(sys.argv[1:])



#!/usr/bin/python
# -*- coding: utf-8 -*-

################################################################################################################
# This script is a simple management system for series made of exercises and solution.                         #
# It is possible to make zipped series for moodle, a zip containing all series. Furthermore one can            #
# generate previews for one exercise/solution. Two handy functions are the make-workbook and the               #
# make-catalogue. The former one creaets a pdf containig all series, each one followed by its solution         #
# just like they were distributed. The latter one create a sort of index of all available exercises in         #
# the system. Each exercise is followed by its solution.                                                       #
#                                                                                                              #
# The structure for a new exercise should be created by using the make-new-exercise function.                  #
# For further help, please refer to the help function of the software.                                         #
# -------------------------------------------------------------------------------------------------------------#
# Author: Andreas Ruppen                                                                                       #                                                                                                                                                             #
# Licensed under the Apache License, Version 2.0 (the "License");                                              #
#   you may not use this file except in compliance with the License.                                           #
#   You may obtain a copy of the License at                                                                    #
#                                                                                                              #
#       http://www.apache.org/licenses/LICENSE-2.0                                                             #
#                                                                                                              #
#   Unless required by applicable law or agreed to in writing, software                                        #
#   distributed under the License is distributed on an "AS IS" BASIS,                                          #
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                   #
#   See the License for the specific language governing permissions and                                        #
#   limitations under the License.                                                                             #
################################################################################################################


import logging
import os
import re
import subprocess
import sys
import datetime
import shutil
from subprocess import STDOUT
try:
    from subprocess import DEVNULL # py3k
except ImportError:
    import os
    DEVNULL = open(os.devnull, 'wb')

log = logging.getLogger('seriesManagementSystem')

def cleanDSStore(dir):
    def cleaner(currdir, dirs, files):
        for item in files:
            if (item.find("DS_Store") != -1):
                log.debug("removing "+currdir+"/"+item)
                os.remove(os.path.join(currdir, item))
    for root, dirs, files in os.walk("./"):
        #self.cleanDSStore(root, dirs, files)
        cleaner(root, dirs, files)

def cleanTempFiles(keepTempFiles):
    if keepTempFiles:
        return
    log.info("Removing temp files")
    tmpfile = os.listdir('/tmp')
    tmpfile.sort()
    for file in tmpfile:
        if file.find("serie") != -1 or file.find("solution") != -1 or file.find("wbtitle")!= -1 or file.find("catalog")!= -1 or file.find("exam")!= -1:
            log.debug("Removing /tmp/"+file)
            os.remove('/tmp/'+file)

def nextUnusedExercice(dir):
    lastExo = 0
    dirs = os.listdir(dir)
    #dirs.sort()
    dirs = natsort(dirs)
    for dir in dirs:
        exoNumber = int(dir.partition('ex')[2])
        if(exoNumber > lastExo):
            lastExo = exoNumber
    lastExo =  int(lastExo)+1
    return lastExo



def natsort(list_):
    log.debug("Will filter the list "+str(list_))
    filteredList = [elem for elem in list_ if not elem.startswith(".")]
    log.debug('Filtered list: '+str(filteredList))
    # decorate
    tmp = [ (int(re.search('\d+', i).group(0)), i) for i in filteredList ]
    log.debug('Tmp list: '+str(tmp))
    tmp.sort()
    #   undecorate
    return [ i[1] for i in tmp ]

def doCheckInstall():
    """Verifies if all needed packets are installed"""
    log.info("Checking dependencies")
    missingProgs = list()
    try:
        subprocess.check_call("which gs", shell=True, stdout=open("/dev/null", 'w'))
    except subprocess.CalledProcessError as e:
        log.error("Did not found ghostscript (gs)")
        log.debug(e)
        missingProgs.append("Ghostscript (gs)")
    try:
        subprocess.check_call("which pdflatex", shell=True, stdout=open("/dev/null", 'w'))
    except subprocess.CalledProcessError as e:
        log.error("Did not found pdflatex")
        log.debug(e)
        missingProgs.append("PdfLaTeX")
    try:
        subprocess.check_call("which latexmk", shell=True, stdout=open("/dev/null", 'w'))
    except subprocess.CalledProcessError as e:
        log.error("Did not found latexmk")
        log.debug(e)
        missingProgs.append("latexmk")
    try:
        subprocess.check_call("which tar", shell=True, stdout=open("/dev/null", 'w'))
    except subprocess.CalledProcessError as e:
        log.error("Did not found Tar utility")
        log.debug(e)
        missingProgs.append("Tar")
    try:
        if len(missingProgs) !=0:
            raise Exception(missingProgs)
    except Exception as x:
        log.error("Please ensure that the needed utilities ("+x.missing+") are installed and on the $PATH")
        sys.exit(-1)

def doLatex(texFile,  outputDir,  doBibTex=False):
    log.info("Running latex in %s on file %s", outputDir, texFile)
    log.debug("LaTeX command is: latexmk -pdf -silent -bibtex- -outdir={:s} {:s}".format(outputDir,  texFile))
    status = subprocess.call(["latexmk", "-pdf",  "-silent",  "-bibtex-", "-outdir="+outputDir,   texFile], cwd="./", stdout=DEVNULL, stderr=STDOUT)
    log.info("Compilation succeded "+texFile)
    # Alternatively use latexmk -c -jobname=texFile plus remove the *.tex file
    tmpfiles = os.listdir(outputDir);
    tmpfiles.sort()
    basename = os.path.split(texFile)[1].split(".")[0]
    for file in tmpfiles:
        if file.find(".pdf")==-1 and file.find(basename)!=-1:
            os.remove(os.path.join(outputDir, file))

def doLatex2(texFile, outputDir, doBibTex=False):
    log.info("Running latex in %s on file %s", outputDir, texFile)
    log.debug("LaTeX command is: pdflatex -output-directory="+outputDir+" "+texFile)
    #Genral settings for latex copmiling
    latex_error_messages = (
                            "Type X to quit or <RETURN> to proceed",
                            "! Undefined control sequence.",
                            "? ",
                            "Type  H <return>  for immediate help.",
                            "Enter file name: ",
                            "or enter new name. (Default extension: sty)"
                            )
    latex_recompile_messages = (
                                "recompile",
                                "re-run",
                                "undefined references",
                                "rerun"
                                )
    auxFile = os.path.split(texFile)[1].split(".")[0]+'.aux'
    logFile = os.path.split(texFile)[1].split(".")[0]+'.log'
    recompile = True
    counter = 0;
    while recompile and counter < 5:
        recompile=False
        counter+=1
        status = subprocess.call(["pdflatex", "-output-directory="+outputDir, "-halt-on-error",  texFile], cwd="./", stdout=open("/dev/null", 'w'))
        if doBibTex:
            bibstatus = subprocess.call(["bibtex", os.path.join(outputDir,auxFile)], cwd="./", stdout=open("/dev/null", 'w'))
        else:
            bibstatus = 0
        if status != 0:
            log.error("Compilation error occured. Try executing by hand pdflatex -output-directory="+outputDir+" -halt-on-error "+texFile)
            exit(1)
        if bibstatus !=0:
            log.error("Compilation error occured. Try executing by hand bibtex "+os.path.join(outputDir,auxFile))
            exit(1)
        log = open(os.path.join(outputDir, logFile), 'r')
        for line in log:
            for msg in latex_recompile_messages:
                if msg in line:
                    recompile=True
                    log.info("Need to recompile "+texFile)
                    break
        log.close()
    log.info("Compilation succeded "+texFile)
    tmpfiles = os.listdir(outputDir);
    tmpfiles.sort()
    basename = os.path.split(texFile)[1].split(".")[0]
    for file in tmpfiles:
        if file.find(".pdf")==-1 and file.find(basename)!=-1:
            os.remove(os.path.join(outputDir, file))

def doUpdateBibTex(bibtexfile, noCiteList):
    """Updates the last visited date of the nocite bibtex items"""
    log.info("Updating BibTex Last visited time stamp")
    bibtex = open(bibtexfile, 'r')
    bibtexnew = open (os.path.join("/tmp/", bibtexfile), 'w')
    start = False
    year = datetime.datetime.now().strftime("%Y")
    month = datetime.datetime.now().strftime("%b")
    day = datetime.datetime.now().strftime("%d")
    dateExpr = re.compile(r"\{\d*\}\{\w+?\}\{\d*\}")
    for line in bibtex:
        if line.find('biburl{') != -1 and start:
            partitions = dateExpr.split(line)
            log.debug(partitions)
            bibtexnew.write(partitions[0]+'{'+day+'}{'+month+'}{'+year+'}'+partitions[1])
        else:
            bibtexnew.write(line)
        for cite in noCiteList:
            if line.find(cite) != -1:
                start=True
        if line.find("}") != -1 and line.find('},')==-1:
            start=False
    bibtex.close()
    bibtexnew.close()
    shutil.copy(os.path.join("/tmp/", bibtexfile), bibtexfile)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from setuptools import setup, find_packages
from distutils.command.install_data import install_data
from pip.req import parse_requirements

def walk_subpkg(name):
    data_files = []
    package_dir = 'SMS'
    for parent, dirs, files in os.walk(os.path.join(package_dir, name)):
        sub_dir = os.sep.join(parent.split(os.sep)[1:])  # remove package_dir from the path
        for f in files:
            data_files.append(os.path.join(sub_dir, f))
    return data_files


cmdclass = {'install_data': install_data}
data_files = [('/etc/SeriesManagementSystem/', ['etc/logging.conf']), ('/usr/local/etc/bash_completion.d/', ['etc/serieManagementSystem-completion.bash'])]
package_data = {'SMS': [] + walk_subpkg('data/')}
# parse_requirements() returns generator of pip.req.InstallRequirement objects
install_reqs = parse_requirements('requirements.txt')
reqs = [str(ir.req) for ir in install_reqs]



# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="seriesManagementSystem",
    version="1.1",
    author="Andreas Ruppen",
    author_email="andreas.ruppen@gmail.com",
    description="Manages Series",
    license="Apache",
    keywords="students series",
    url="https://github.com/digsim/seriesManagementSystem",
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    package_data = package_data,
    entry_points={
        'console_scripts': [
            'seriesManagementSystem=SMS:main',
        ],
    },
    cmdclass=cmdclass,
    data_files=data_files,
    install_requires=reqs,
    long_description=read('README.md'),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        'Intended Audience :: Education',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Science/Research',
        "Topic :: Utilities",
        'Topic :: Education',
        'Topic :: Software Development :: Compilers',
        "License :: OSI Approved :: Apache Software License",
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    platforms='any',
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

################################################################################################################
# This script is a simple management system for series made of exercises and solution.                         #
# It is possible to make zipped series for moodle, a zip containing all series. Furthermore one can            #
# generate previews for one exercise/solution. Two handy functions are the make-workbook and the               #
# make-catalogue. The former one creaets a pdf containig all series, each one followed by its solution         #
# just like they were distributed. The latter one create a sort of index of all available exercises in         #
# the system. Each exercise is followed by its solution.                                                       #
#                                                                                                              #
# The structure for a new exercise should be created by using the make-new-exercise function.                  #
# For further help, please refer to the help function of the software.                                         #
# -------------------------------------------------------------------------------------------------------------#
# Author: Andreas Ruppen                                                                                       #                                                                                                                                                             #
# Licensed under the Apache License, Version 2.0 (the "License");                                              #
#   you may not use this file except in compliance with the License.                                           #
#   You may obtain a copy of the License at                                                                    #
#                                                                                                              #
#       http://www.apache.org/licenses/LICENSE-2.0                                                             #
#                                                                                                              #
#   Unless required by applicable law or agreed to in writing, software                                        #
#   distributed under the License is distributed on an "AS IS" BASIS,                                          #
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                   #
#   See the License for the specific language governing permissions and                                        #
#   limitations under the License.                                                                             #
################################################################################################################

import logging
import os
import zipfile
import tarfile
import subprocess
import shutil

log = logging.getLogger('seriesManagementSystem')

def myZip(directory, destZipFile, zipPrefix="."):
    """Zips a directory recursively to the destination zipfile"""
    log.debug("Zipping directory: "+directory+" to "+destZipFile)
    if len(os.listdir(directory)) == 0:
        return
    zippedDir = zipfile.ZipFile(destZipFile, 'w')
    def zipTreeWalker(args, dirname, fnames):
        theZipArch = args[0]
        root = args[1]
        prefix = args[2]
        fnames.sort()
        for file in fnames:
            file = os.path.join(dirname, file)
            archiveName = file[len(os.path.commonprefix((root, file)))+1:]
            archiveName = os.path.join(prefix, archiveName)
            if not os.path.isdir(file):
                theZipArch.write(file, archiveName)
    for root, dirs, files in os.walk(directory):
        zipTreeWalker([zippedDir, directory, zipPrefix], root, files)
    

def myTar(directory, destTarFile, tarPrefix="."):
    """Creates a tar.gz file of a directory to destTarFile"""
    def isSvn(f):
        return f.endswith(".svn")
    
    log.debug("Tar.gz - ing "+directory+" to "+destTarFile+". Using python tar")
    containingFolder = os.path.basename(destTarFile)[:os.path.basename(destTarFile).find(".")]
    tarTempName = "/tmp/tmp.tar.gz"
    files = os.listdir(directory)
    if len(files) == 0:
        return
    tarArchive = tarfile.open(tarTempName, 'w:gz')
    cwd = os.getcwd()
    os.chdir(os.path.join(cwd, directory))
    for file in files:
        tarArchive.add(file, containingFolder+"/"+file, exclude=isSvn)
    os.chdir(cwd)
    if len(tarArchive.getmembers()) == 0: 
        return
    tarArchive.close()
    
    shutil.move(tarTempName, destTarFile)
    return destTarFile

def sysTar(directory, destTarFile, tarPrefix="."):
    cwd = os.getcwd()
    log.debug("Tar.gz - ing "+directory+" to "+destTarFile+". Using system tar")
    tarTempName = "/tmp/tmp.tar.gz"
    basename = os.path.basename(directory)
    dirname = os.path.dirname(directory)
    os.chdir(os.path.join(cwd, dirname))
    subprocess.call(["tar -czf "+tarTempName+" --exclude='\.svn' "+basename], shell=True, cwd="./", stdout=open("/dev/stdout", 'w'))
    os.chdir(cwd)
    shutil.move(tarTempName, destTarFile)
        


#!/usr/bin/python
# -*- coding: utf-8 -*-

################################################################################################################
# This script is a simple management system for series made of exercises and solution.                         #
# It is possible to make zipped series for moodle, a zip containing all series. Furthermore one can            #
# generate previews for one exercise/solution. Two handy functions are the make-workbook and the               #
# make-catalogue. The former one creaets a pdf containig all series, each one followed by its solution         #
# just like they were distributed. The latter one create a sort of index of all available exercises in         #
# the system. Each exercise is followed by its solution.                                                       #
#                                                                                                              #
# The structure for a new exercise should be created by using the make-new-exercise function.                  #
# For further help, please refer to the help function of the software.                                         #
# -------------------------------------------------------------------------------------------------------------#
# Author: Andreas Ruppen                                                                                       #                                                                                                                                                             #
# Licensed under the Apache License, Version 2.0 (the "License");                                              #
#   you may not use this _file except in compliance with the License.                                           #
#   You may obtain a copy of the License at                                                                    #
#                                                                                                              #
#       http://www.apache.org/licenses/LICENSE-2.0                                                             #
#                                                                                                              #
#   Unless required by applicable law or agreed to in writing, software                                        #
#   distributed under the License is distributed on an "AS IS" BASIS,                                          #
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                   #
#   See the License for the specific language governing permissions and                                        #
#   limitations under the License.                                                                             #
################################################################################################################

import logging
import sys
if float(sys.version[:3])<3.0:
    import ConfigParser
else:
    import configparser as ConfigParser
from pkg_resources import resource_filename
from os.path import dirname, join

class LaTeX:
    
    def __init__(self, serie):
        """initialization stuff"""
        smsConfig = ConfigParser.SafeConfigParser()
        smsConfig.read([join(resource_filename(__name__, 'data'), 'lecture.cfg'), "lecture.cfg"])


        self.__name = smsConfig.get("Lecture", "name")
        self.__lecturer = smsConfig.get("Lecture", "lecturer")
        self.__year = smsConfig.get("Lecture", "year")
        self.__exercisetext = smsConfig.get("Lecture", "exercisetext")
        self.__solutiontext = smsConfig.get("Lecture", "solutiontext")
        self.__contenttext = smsConfig.get("Lecture", "contenttext")
        self.__headertitle = smsConfig.get("Lecture", "headertitle")
        self.__bibtex = smsConfig.get("Lecture", "bibtex")
        self.__noCiteList = smsConfig.get("Lecture", "nocite").split(",")
        self.__unilogo = smsConfig.get("Logo", "unilogo")
        self.__groupelogo = smsConfig.get("Logo", "groupelogo")
        self.__pdfkeyword = smsConfig.get("PDF", "pdfkeyword")
        self.__pdftitle = smsConfig.get("PDF", "pdftitle")
        self.__pdfauthor = smsConfig.get("PDF", "pdfauthor")

        self.__serie = serie
        self.__log = logging.getLogger('seriesManagementSystem')


    def createHeader(self, _file, titles):
        _file.write(r'\documentclass[francais,a4paper]{article}'+"\n")
        _file.write(r'\usepackage{sms}'+"\n")
        _file.write(r"\newcommand{\compilationpath}{./}"+"\n")
        _file.write(r'\newcommand{\prof}{'+self.__lecturer+'}'+"\n")
        _file.write(r'\newcommand{\course}{'+self.__name+'}'+"\n")
        _file.write(r'\newcommand{\theyear}{'+self.__year+'}'+"\n")
        _file.write(r'\newcommand{\exercisetext}{'+self.__exercisetext+'}'+"\n")
        _file.write(r'\newcommand{\solutiontext}{'+self.__solutiontext+'}'+"\n")
        _file.write(r'\newcommand{\thecontent} {\sffamily\bfseries '+self.__contenttext+':}'+"\n")
        _file.write(r'\newcommand{\theheadertitle}{'+self.__headertitle+'}'+"\n")
        _file.write(r'\newcommand{\unilogo}{'+self.__unilogo+'}'+"\n")
        _file.write(r'\newcommand{\groupelogo}{'+self.__groupelogo+'}'+"\n")
        #afile.write(r"\input{\compilationpath/exercicepreamble}"+"\n")
        _file.write(r"% Number of the serie"+"\n")
        _file.write(r"\newcommand{\exercisenb}{"+str(self.__serie)+"}"+"\n")
        _file.write(r"\newcommand{\includepath}{\compilationpath}"+"\n")
        _file.write(r'\hypersetup{pdftitle={'+self.__pdftitle+'},pdfauthor={'+self.__pdfauthor+'},pdfkeywords={'+self.__pdfkeyword+"}}\n")
        _file.write(r"\begin{document}"+"\n")
        _file.write(r"\input{\compilationpath/captionnames}"+"\n")
        _file.write(r"% Header of the exercise:"+"\n")
        _file.write(r"\exheader"+"\n")

        if len(titles) != 0:
            _file.write(r"% Content of the exercise, topics"+"\n")
            _file.write(r"\content{"+"\n")
            _file.write(r"\begin{itemize}"+"\n")
            for title in titles:
                _file.write(r'\item '+title+'\n')
            _file.write(r'\end{itemize}'+'\n')
            _file.write(r'}'+'\n')
    
    def createFooter(self, _file):
        for bib in self.__noCiteList:
            _file.write(r'\nocite{'+bib+'}\n')
        _file.write(r'\bibliography{bibdb}'+'\n')
        _file.write(r'\bibliographystyle{plain}'+'\n')
        _file.write(r'\end{document}'+'\n')



    def makeWorkBookTitlePageHeader(self, _file):
        _file.write(r"\documentclass[francais,a4paper]{article}"+"\n")
        _file.write(r"\newcommand{\compilationpath}{./}"+"\n")
        _file.write(r'\newcommand{\groupelogo}{'+self.__groupelogo+'}'+"\n")
        _file.write(r"\usepackage{graphicx}"+"\n")
        _file.write(r"\usepackage{palatino}"+"\n")
        _file.write(r"%\usepackage[french]{babel}"+"\n")
        _file.write(r"\usepackage[utf8]{inputenc}"+"\n")
        _file.write(r"\usepackage{ae, pslatex}    % Joli output en PDF"+"\n")
        _file.write(r"%\usepackage{graphics}          % Manipulation de boîtes et importation de graphismes."+"\n")
        _file.write(r"%\usepackage[dvips]{graphicx}   %"+"\n")
        _file.write(r"\usepackage[T1]{fontenc}"+"\n")
        _file.write(r"\begin{document}"+"\n")
        _file.write(r"\pagestyle{empty}"+"\n")
        _file.write(r"\vspace{-1cm}"+"\n")
        _file.write(r"\begin{center}"+"\n")
        _file.write(r"\begin{Huge}"+"\n")
        _file.write(r"{\sf "+self.__name+" }"+"\n")
        _file.write(r"\end{Huge}"+"\n")
        _file.write(r"\vspace{0.4cm}%"+"\n")
        _file.write(r"\begin{huge}"+"\n")
        _file.write(r"Workbook ("+self.__year+")"+"\n")
        _file.write(r"\end{huge}"+"\n")
        _file.write(r"\end{center}"+"\n")
        _file.write(r"\rule{\linewidth}{1pt}"+"\n")
        _file.write(r"\vspace{1cm}"+"\n")

    def printWorkBookTitlePageFooter(self, _file):
        _file.write(r"%\end{itemize}"+"\n")
        _file.write(r"\rule{\linewidth}{1pt}"+"\n")
        _file.write(r"\vfill"+"\n")
        _file.write(r"\centering"+"\n")
        _file.write(r"\includegraphics[height=1.65cm]{\compilationpath/logos/\groupelogo}"+"\n")
        _file.write(r"\end{document}"+"\n")
        
